a{
	color: #0969da;
	text-decoration: none;
}
body{
	font-size: 14px;
	font-family: -apple-system,BlinkMacSystemFont,"Segoe UI",Helvetica,Arial,sans-serif,"Apple Color Emoji","Segoe UI Emoji";
	line-height: 1.5;
	word-wrap: break-word;
}

h1 {
	color: #641818;
	border-bottom: 1px solid #d0d0d0;
	font-size: 25px;
	line-height: 38px;
}

h2 {
	color: #641818;
	border-bottom: 1px dashed #d0d0d0;
	font-size: 23px;
	line-height: 30px;
}


h3 {
	color: #353f6e;
	font-size: 20px;
}


h4 {
	color: #641818;
	font-size: 18px;
}


h5 {
	color: #641818;
	font-size: 16px;
}

h6 {
	color: #641818;
	font-size: 15px;
}

.layout{
	display: grid;
	grid-auto-flow: column;
	grid-template-columns: auto 0 1fr;
	grid-gap: 24px;
	padding-right: 32px;
	padding-left: 32px;
	max-width: 60em;
	margin-right: auto;
	margin-left: auto;
	margin-top: 24px;
}
.layout-sidebar{
	grid-column: 1;
	width:15em;
	border: 1px solid #dfdfdf;
	border-top: none;
	padding: 5px 10px;	
	margin-bottom: 5px;	
	background: #f4f4f4;
}
.layout-main{
	grid-column: 2/span 2;
	max-width: 42em;
}

.layout-sidebar ul {
	margin: 0;
	border: none;
	padding-left: 15px;
}

.layout-sidebar li {
	background-repeat: no-repeat;
	background-position: left center;
	display: inline-block;
	list-style: none;
	font-size: 13px;
	/* padding-left: 20px; */
	padding-right: 10px;
	line-height: 10px;
	border-right: 1px dotted #d4d4d4;
}

.layout-sidebar li.current a {
	color: black;
}

.fa-folder{
	color: #dcb67a;
	padding-right: 5px;
}

.fa-file{
	color: #c9c3b9;
	padding-right: 5px;
}

.path {
	border: 1px solid #dfdfdf;
	padding: 5px 10px;	
	background: #eaeaea;
}

.path a {
	/* padding-left: 20px;  */
	padding-right: 5px;
	background-repeat: no-repeat;
	background-position: left center;
}

.path a:last-child {
	color: black;
}